@@ -361,7 +361,7 @@ rbitshift_component( register CARD32 *src, register CARD32 *dst, int shift, int
 #endif
 
 static inline void
-start_component_interpolation( CARD32 *c1, CARD32 *c2, CARD32 *c3, CARD32 *c4, register CARD32 *T, register CARD32 *step, CARD16 S, int len)
+start_component_interpolation( CARD32 *c1, CARD32 *c2, CARD32 *c3, CARD32 *c4, register CARD32 *T, register CARD32 *step, int S, int len)
 {
 	register int i;
 	for( i = 0 ; i < len ; i++ )
@@ -624,7 +624,7 @@ make_gradient_scanline( ASScanline *scl, ASGradient *grad, ASFlagType filter, AR
 /* Scaling code ; 																			   */
 /* **********************************************************************************************/
 Bool
-check_scale_parameters( ASImage *src, unsigned int src_width, unsigned int src_height, unsigned int *to_width, unsigned int *to_height )
+check_scale_parameters( ASImage *src, int src_width, int src_height, int *to_width, int *to_height )
 {
 	if( src == NULL )
 		return False;
@@ -772,7 +772,7 @@ scale_image_up( ASImageDecoder *imdec, ASImageOutput *imout, int h_ratio, int *s
                     imout->output_image_scanline( imout, c1, 1);
                 }else
                 {
-                    SCANLINE_COMBINE(start_component_interpolation,*c1,*c2,*c3,*c4,*c1,step,(CARD8)S,out_width);
+                    SCANLINE_COMBINE(start_component_interpolation,*c1,*c2,*c3,*c4,*c1,step,S,out_width);
                     do
                     {
                         imout->output_image_scanline( imout, c1, 1);
@@ -835,7 +835,7 @@ create_destination_image( unsigned int width, unsigned int height, ASAltImFormat
 /* ASImage transformations : 												  */
 /* *****************************************************************************/
 ASImage *
-scale_asimage( ASVisual *asv, ASImage *src, unsigned int to_width, unsigned int to_height,
+scale_asimage( ASVisual *asv, ASImage *src, int to_width, int to_height,
 			   ASAltImFormats out_format, unsigned int compression_out, int quality )
 {
 	ASImage *dst = NULL ;
@@ -910,9 +910,9 @@ scale_asimage( ASVisual *asv, ASImage *src, unsigned int to_width, unsigned int
 
 ASImage *
 scale_asimage2( ASVisual *asv, ASImage *src, 
-					unsigned int clip_x, unsigned int clip_y, 
-					unsigned int clip_width, unsigned int clip_height, 
-					unsigned int to_width, unsigned int to_height,
+					int clip_x, int clip_y, 
+					int clip_width, int clip_height, 
+					int to_width, int to_height,
 			   		ASAltImFormats out_format, unsigned int compression_out, int quality )
 {
 	ASImage *dst = NULL ;
@@ -994,8 +994,8 @@ scale_asimage2( ASVisual *asv, ASImage *src,
 ASImage *
 tile_asimage( ASVisual *asv, ASImage *src,
 		      int offset_x, int offset_y,
-			  unsigned int to_width,
-			  unsigned int to_height,
+			  int to_width,
+			  int to_height,
 			  ARGB32 tint,
 			  ASAltImFormats out_format, unsigned int compression_out, int quality )
 {
@@ -1060,8 +1060,8 @@ LOCAL_DEBUG_OUT("tiling actually...%s", "");
 ASImage *
 merge_layers( ASVisual *asv,
 				ASImageLayer *layers, int count,
-			  	unsigned int dst_width,
-			  	unsigned int dst_height,
+			  	int dst_width,
+			  	int dst_height,
 			  	ASAltImFormats out_format, unsigned int compression_out, int quality )
 {
 	ASImage *fake_bg = NULL ;
@@ -1075,13 +1075,13 @@ merge_layers( ASVisual *asv,
 
 LOCAL_DEBUG_CALLER_OUT( "dst_width = %d, dst_height = %d", dst_width, dst_height );
 	
-	prepare_scanline( dst_width, QUANT_ERR_BITS, &dst_line, asv->BGR_mode );
-	dst_line.flags = SCL_DO_ALL ;
-
 	dst = create_destination_image( dst_width, dst_height, out_format, compression_out, ARGB32_DEFAULT_BACK_COLOR );
 	if( dst == NULL )
 		return NULL;
 
+	prepare_scanline( dst_width, QUANT_ERR_BITS, &dst_line, asv->BGR_mode );
+	dst_line.flags = SCL_DO_ALL ;
+
 	imdecs = safecalloc( count+20, sizeof(ASImageDecoder*));
 
 /*  don't really know why the hell we need that ???
@@ -1125,6 +1125,7 @@ LOCAL_DEBUG_CALLER_OUT( "dst_width = %d, dst_height = %d", dst_width, dst_height
 				stop_image_decoding( &(imdecs[i]) );
 
         destroy_asimage( &dst );
+		free_scanline( &dst_line, True );
     }else
 	{
 		int y, max_y = 0;
@@ -1137,7 +1138,7 @@ LOCAL_DEBUG_OUT("blending actually...%s", "");
 		{
 			if( imdecs[i] )
 			{
-				unsigned int layer_bottom = pcurr->dst_y+pcurr->clip_height ;
+				int layer_bottom = pcurr->dst_y+pcurr->clip_height ;
 				if( pcurr->dst_y < min_y )
 					min_y = pcurr->dst_y;
 				layer_bottom += imdecs[i]->bevel_v_addon ;
@@ -1305,8 +1306,8 @@ make_gradient_diag_width( ASImageOutput *imout, ASScanline *dither_lines, int di
 {
 	int line = 0;
 	/* using bresengham algorithm again to trigger horizontal shift : */
-	unsigned short smaller = imout->im->height;
-	unsigned short bigger  = imout->im->width;
+	short smaller = imout->im->height;
+	short bigger  = imout->im->width;
 	register int i = 0;
 	int eps;
 LOCAL_DEBUG_CALLER_OUT( "width = %d, height = %d, filetr = 0x%lX, dither_count = %d, dither width = %d\n", bigger, smaller, filter, dither_lines_num, dither_lines[0].width );
@@ -1441,7 +1442,7 @@ get_best_grad_back_color( ASGradient *grad )
 
 ASImage*
 make_gradient( ASVisual *asv, ASGradient *grad,
-               unsigned int width, unsigned int height, ASFlagType filter,
+               int width, int height, ASFlagType filter,
   			   ASAltImFormats out_format, unsigned int compression_out, int quality  )
 {
 	ASImage *im = NULL ;
@@ -1516,8 +1517,8 @@ LOCAL_DEBUG_CALLER_OUT( "type = 0x%X, width=%d, height = %d, filter = 0x%lX", gr
 ASImage *
 flip_asimage( ASVisual *asv, ASImage *src,
 		      int offset_x, int offset_y,
-			  unsigned int to_width,
-			  unsigned int to_height,
+			  int to_width,
+			  int to_height,
 			  int flip,
 			  ASAltImFormats out_format, unsigned int compression_out, int quality )
 {
@@ -1527,9 +1528,10 @@ flip_asimage( ASVisual *asv, ASImage *src,
 	START_TIME(started);
 
 LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height = %d", offset_x, offset_y, to_width, to_height );
-	if( src )
-		filter = get_asimage_chanmask(src);
-
+	if( src == NULL )
+		return NULL ;
+	
+	filter = get_asimage_chanmask(src);
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
 
 #ifdef HAVE_MMX
@@ -1635,8 +1637,8 @@ LOCAL_DEBUG_OUT("flip-flopping actually...%s", "");
 ASImage *
 mirror_asimage( ASVisual *asv, ASImage *src,
 		      int offset_x, int offset_y,
-			  unsigned int to_width,
-			  unsigned int to_height,
+			  int to_width,
+			  int to_height,
 			  Bool vertical, ASAltImFormats out_format,
 			  unsigned int compression_out, int quality )
 {
@@ -1699,8 +1701,8 @@ LOCAL_DEBUG_OUT("miroring actually...%s", "");
 ASImage *
 pad_asimage(  ASVisual *asv, ASImage *src,
 		      int dst_x, int dst_y,
-			  unsigned int to_width,
-			  unsigned int to_height,
+			  int to_width,
+			  int to_height,
 			  ARGB32 color,
 			  ASAltImFormats out_format,
 			  unsigned int compression_out, int quality )
@@ -1768,7 +1770,7 @@ LOCAL_DEBUG_OUT( "filling %d lines with %8.8lX", start_y, color );
 			result.back_color = imdec->buffer.back_color ;
 		if( (int)to_width == clip_width )
 		{
-			if( (int)clip_width == (int)src->width )
+			if( imdec == NULL )
 			{
 LOCAL_DEBUG_OUT( "copiing %d lines", clip_height );
 				copy_asimage_lines( dst, start_y, src, (dst_y < 0 )? -dst_y: 0, clip_height, SCL_DO_ALL );
@@ -1779,7 +1781,7 @@ LOCAL_DEBUG_OUT( "copiing %d lines", clip_height );
 					imdec->decode_image_scanline( imdec );
 					imout->output_image_scanline( imout, &(imdec->buffer), 1);
 				}
-		}else
+		}else if( imdec )
 		{
 			for( y = 0 ; y < clip_height ; y++  )
 			{
@@ -1812,7 +1814,7 @@ LOCAL_DEBUG_OUT( "filling %d lines with %8.8lX at the end", to_height-(start_y+c
 		for( y = start_y+clip_height ; y < (int)to_height ; y++  )
 			imout->output_image_scanline( imout, &result, 1);
 
-		if( (int)to_width != clip_width || clip_width != (int)src->width )
+		if( imdec )
 		{
 			stop_image_decoding( &imdec );
 			free_scanline( &result, True );
@@ -2012,7 +2014,7 @@ colorize_asimage_vector( ASVisual *asv, ASImage *im,
 
 ASImage *
 create_asimage_from_vector( ASVisual *asv, double *vector,
-							unsigned int width, unsigned int height,
+							int width, int height,
 							ASVectorPalette *palette,
 							ASAltImFormats out_format,
 							unsigned int compression, int quality )
@@ -2172,8 +2174,8 @@ static void calc_gauss(double radius, double* gauss) {
 ASImage*
 adjust_asimage_hsv( ASVisual *asv, ASImage *src,
 				    int offset_x, int offset_y,
-	  			    unsigned int to_width, unsigned int to_height,
-					unsigned int affected_hue, unsigned int affected_radius,
+	  			    int to_width, int to_height,
+					int affected_hue, int affected_radius,
 					int hue_offset, int saturation_offset, int value_offset,
 					ASAltImFormats out_format,
 					unsigned int compression_out, int quality )
@@ -2184,7 +2186,9 @@ adjust_asimage_hsv( ASVisual *asv, ASImage *src,
 	START_TIME(started);
 
 LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height = %d, hue = %u", offset_x, offset_y, to_width, to_height, affected_hue );
-	if( src && (imdec = start_image_decoding(asv, src, SCL_DO_ALL, offset_x, offset_y, to_width, 0, NULL)) == NULL )
+	if( src == NULL ) 
+		return NULL;
+	if( (imdec = start_image_decoding(asv, src, SCL_DO_ALL, offset_x, offset_y, to_width, 0, NULL)) == NULL )
 		return NULL;
 
 	dst = create_destination_image( to_width, to_height, out_format, compression_out, src->back_color);
@@ -2199,9 +2203,12 @@ LOCAL_DEBUG_CALLER_OUT( "offset_x = %d, offset_y = %d, to_width = %d, to_height
 	{
 	    CARD32 from_hue1 = 0, from_hue2 = 0, to_hue1 = 0, to_hue2 = 0 ;
 		int y, max_y = to_height;
+		Bool do_greyscale = False ; 
 
 		affected_hue = normalize_degrees_val( affected_hue );
 		affected_radius = normalize_degrees_val( affected_radius );
+		if( value_offset != 0 )
+			do_greyscale = (affected_hue+affected_radius >= 360 || affected_hue-affected_radius <= 0 );
 		if( affected_hue > affected_radius )
 		{
 			from_hue1 = degrees2hue16(affected_hue-affected_radius);
@@ -2270,6 +2277,10 @@ LOCAL_DEBUG_OUT("adjusting actually...%s", "");
 						fprintf( stderr, "OUT %d: rgb = #%4.4lX.%4.4lX.%4.4lX hue = %ld(%ld)     sat = %ld val = %ld\n", __LINE__, r[x], g[x], b[x], h, ((h>>8)*360)>>8, s, v );
 #endif
 					}
+				}else if( do_greyscale ) 
+				{
+					int tmp = (int)r[x] + value_offset ; 
+					g[x] = b[x] = r[x] = (tmp < 0)?0:((tmp>0x00FFFF)?0x00FFff:tmp);
 				}
 			}
 			imdec->buffer.flags = 0xFFFFFFFF ;
@@ -2355,10 +2366,10 @@ slice_scanline( ASScanline *dst, ASScanline *src, int start_x, int end_x, ASScan
 
 ASImage*
 slice_asimage2( ASVisual *asv, ASImage *src,
-			   unsigned int slice_x_start, unsigned int slice_x_end,
-			   unsigned int slice_y_start, unsigned int slice_y_end,
-			   unsigned int to_width,
-			   unsigned int to_height,
+			   int slice_x_start, int slice_x_end,
+			   int slice_y_start, int slice_y_end,
+			   int to_width,
+			   int to_height,
 			   Bool scale,
 			   ASAltImFormats out_format,
 			   unsigned int compression_out, int quality )
@@ -2550,6 +2561,7 @@ LOCAL_DEBUG_CALLER_OUT( "sx1 = %d, sx2 = %d, sy1 = %d, sy2 = %d, to_width = %d,
 				imout->output_image_scanline( imout, out_buf, 1);
 			}
 		}
+		free_scanline( out_buf, False );
 		stop_image_output( &imout );
 	}
 #ifdef HAVE_MMX
@@ -2563,10 +2575,9 @@ LOCAL_DEBUG_CALLER_OUT( "sx1 = %d, sx2 = %d, sy1 = %d, sy2 = %d, to_width = %d,
 
 ASImage*
 slice_asimage( ASVisual *asv, ASImage *src,
-			   unsigned int slice_x_start, unsigned int slice_x_end,
-			   unsigned int slice_y_start, unsigned int slice_y_end,
-			   unsigned int to_width,
-			   unsigned int to_height,
+			   int slice_x_start, int slice_x_end,
+			   int slice_y_start, int slice_y_end,
+			   int to_width, int to_height,
 			   ASAltImFormats out_format,
 			   unsigned int compression_out, int quality )
 {
