@@ -265,8 +265,6 @@ main (int argc, char **argv)
                     M_END_WINDOWLIST |
                     WINDOW_CONFIG_MASK |
                     WINDOW_NAME_MASK, 0);
-    balloon_init (False);
-
     Config = CreateWharfConfig ();
 
     LOCAL_DEBUG_OUT("parsing Options ...%s","");
@@ -348,8 +346,8 @@ DeadPipe (int nonsense)
 	destroy_wharf_folder( &(WharfState.root_folder) );
     DestroyWharfConfig( Config );
     destroy_ashash( &(WharfState.win2obj_xref) );
+    destroy_ashash( &(WharfState.swallow_targets) );
     window_data_cleanup();
-
     FreeMyAppResources();
 #ifdef DEBUG_ALLOCS
 /* normally, we let the system clean up, but when auditing time comes
@@ -465,7 +463,7 @@ CheckConfigSanity()
     PrintWharfConfig(Config);
     Print_balloonConfig ( Config->balloon_conf );
 #endif
-    balloon_config2look( &(Scr.Look), Config->balloon_conf, "*WharfBalloon" );
+    balloon_config2look( &(Scr.Look), NULL, Config->balloon_conf, "*WharfBalloon" );
     set_balloon_look( Scr.Look.balloon_look );
 
 }
@@ -522,6 +520,7 @@ SHOW_CHECKPOINT;
 	ASCF_MERGE_SCALAR_KEYWORD(WHARF, to, from, Bevel);
 	ASCF_MERGE_SCALAR_KEYWORD(WHARF, to, from, CompositionMethod);
 	ASCF_MERGE_SCALAR_KEYWORD(WHARF, to, from, FolderOffset);
+	ASCF_MERGE_SCALAR_KEYWORD(WHARF, to, from, OrthogonalFolderOffset);
 
 /*LOCAL_DEBUG_OUT( "align_contents = %d", Config->align_contents ); */
     if( get_flags( config->set_flags, WHARF_SOUND ) )
@@ -547,6 +546,7 @@ SHOW_CHECKPOINT;
         ProcessMyStyleDefinitions (&(config->style_defs));
 
     DestroyWharfConfig (config);
+	FreeSyntaxHash( &WharfFolderSyntax );
     SHOW_TIME("Config parsing",option_time);
 }
 /*****************************************************************************
@@ -790,6 +790,7 @@ DispatchEvent (ASEvent * event)
 				LOCAL_DEBUG_OUT( "AS Styles updated!%s","");
 				mystyle_list_destroy_all(&(Scr.Look.styles_list));
 				LoadColorScheme();
+				reload_asimage_manager( Scr.image_manager );
 				CheckConfigSanity();
 				/* now we need to update everything */
 				update_wharf_folder_styles( WharfState.root_folder, True );
@@ -1375,6 +1376,12 @@ update_wharf_button_styles( ASWharfButton *aswb, Bool odd )
 	if( unfocused_style == NULL ) 
 		unfocused_style = Scr.Look.MSWindow[UNFOCUSED_TILE_STYLE];
 	set_flags( aswb->flags, ASW_Focusable);
+
+	if( get_flags( Config->flags, WHARF_StretchBackground ) )
+		set_flags( aswb->bar->state, BAR_FLAGS_CROP_BACK );
+	else
+		clear_flags( aswb->bar->state, BAR_FLAGS_CROP_BACK );
+
 	if( focused_style == NULL ) 
 	{	
 		if( odd && unfocused_style == Scr.Look.MSWindow[UNFOCUSED_TILE_STYLE] ) 
@@ -1406,7 +1413,7 @@ update_wharf_folder_styles( ASWharfFolder *aswf, Bool force )
 				render_wharf_button( aswb );
 				update_canvas_display( aswb->canvas );
 			}
-			if( aswb->folder && get_flags( aswb->folder->flags, ASW_Mapped )  )
+			if( aswb->folder /*&& get_flags( aswb->folder->flags, ASW_Mapped )*/  )
 				update_wharf_folder_styles( aswb->folder, force );
 		}
 		update_wharf_folder_shape( aswf );
@@ -1433,12 +1440,13 @@ map_wharf_folder( ASWharfFolder *aswf,
 	aswf->gravity = gravity ;
 
 	extwm_hints.pid = getpid();
-    extwm_hints.flags = EXTWM_PID|EXTWM_StateSkipTaskbar|EXTWM_TypeDock|EXTWM_TypeASModule ;
-
+    extwm_hints.flags = EXTWM_PID|EXTWM_TypeSet|EXTWM_StateSet ;
+	extwm_hints.type_flags = EXTWM_TypeDock|EXTWM_TypeASModule ;
+	extwm_hints.state_flags = EXTWM_StateSkipTaskbar ;
     if( aswf != WharfState.root_folder )
 	{
         XSetTransientForHint(dpy, aswf->canvas->w, WharfState.root_folder->canvas->w);
-		extwm_hints.flags |=  EXTWM_TypeDialog ;
+		set_flags(extwm_hints.type_flags,  EXTWM_TypeDialog ) ;
     }else
         protocols = AS_DoesWmDeleteWindow ;
 
@@ -1853,12 +1861,16 @@ display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bot
         y = south? top - height : bottom ;
         if( top != bottom )
             y += south?-Config->FolderOffset:Config->FolderOffset ;
+        if( left != right)
+            x += east?-Config->OrthogonalFolderOffset:Config->OrthogonalFolderOffset ;
     }else
     {
         x = east? left - width : right ;
         y = south? bottom - height: top ;
         if( left != right)
             x += east?-Config->FolderOffset:Config->FolderOffset ;
+        if( top != bottom )
+            y += south?-Config->OrthogonalFolderOffset:Config->OrthogonalFolderOffset ;
     }
 	LOCAL_DEBUG_OUT("calculated pos(%+d%+d), east(%d), south(%d), total_size(%dx%d)", x, y, east, south, total_width, total_height );
 	if( east )
@@ -2144,7 +2156,7 @@ grab_swallowed_canvas_btns( ASCanvas *canvas, ASWharfButton *aswb, Bool withdraw
 void
 update_wharf_folder_size( ASWharfFolder *aswf)
 {
-    unsigned int total_width = 1, total_height = 1;
+    int total_width = 1, total_height = 1;
     if( !get_flags( aswf->flags, ASW_Mapped) )
 		return;		
 	place_wharf_buttons( aswf, &total_width, &total_height );
@@ -2243,7 +2255,7 @@ check_swallow_window( ASWindowData *wd )
     sleep_a_millisec(100);
 	grab_swallowed_canvas_btns( nc, aswb, withdraw_btn );
 
-    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) )
+    if( get_flags( wd->client_icon_flags, AS_ClientIcon ) && !get_flags( wd->client_icon_flags, AS_ClientIconPixmap) )
 		icon_w = wd->icon ;
 	else if( get_flags( wd->flags, AS_WMDockApp ) ) 
 	{
@@ -2312,7 +2324,7 @@ check_swallow_window( ASWindowData *wd )
                                       get_flags(Config->AlignContents,PAD_BOTTOM),
                                       aswb->canvas->height, sheight    ),
                        swidth, sheight );
-	if( !get_flags( wd->flags, AS_ClientIcon ) ) 
+	if( !get_flags( wd->client_icon_flags, AS_ClientIcon ) ) 
 	{  /* workaround for broken wmdock apps that draw into icon, that is a child of a client, 
 		* and at the same time not properly sized ( Just don't ask - some ppl are wierd ) */
 		XWMHints *wm_hints = XGetWMHints (dpy, wd->client);
@@ -2557,7 +2569,10 @@ Bool render_wharf_button( ASWharfButton *aswb )
 		Scr.RootClipArea = aswf->root_clip_area ; 
 	}
 	LOCAL_DEBUG_OUT( "rendering button %p for folder %p. Root Image = %p", aswb, aswf, Scr.RootImage );
-	result = render_astbar( aswb->bar, aswb->canvas );
+	if( get_flags( aswb->bar->state, BAR_FLAGS_CROP_BACK ) )
+		result = render_astbar_cached_back( aswb->bar, aswb->canvas, NULL, aswf->canvas );
+	else
+		result = render_astbar( aswb->bar, aswb->canvas );
 	ASSync( False );
 	--WharfState.buttons_render_pending ;
 
