@@ -158,7 +158,7 @@ SelectDecor (ASWindow * t)
  *
  ***********************************************************************/
 ASWindow     *
-AddWindow (Window w)
+AddWindow (Window w, Bool from_map_request)
 {
 	ASWindow     *tmp_win;					   /* new afterstep window structure */
 	ASRawHints    raw_hints ;
@@ -188,7 +188,7 @@ AddWindow (Window w)
     {
         if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
             print_hints( NULL, NULL, &raw_hints );
-        hints = merge_hints( &raw_hints, Database, &status, Scr.Look.supported_hints, HINT_ANY, NULL );
+        hints = merge_hints( &raw_hints, Database, &status, Scr.Look.supported_hints, HINT_ANY, NULL, w );
         destroy_raw_hints( &raw_hints, True );
         if( hints )
         {
@@ -264,6 +264,8 @@ AddWindow (Window w)
 	ungrab_server();
 
 	/* doing that for any window seems to resolve java sizing bugs */
+	/* this will get called from set_window_wm_state() again just few lines below
+	   which is a bit of a excess, but we don't seem to have much choice */
     on_window_status_changed( tmp_win, False, True );
     if( pending_placement )
     {
@@ -277,11 +279,14 @@ AddWindow (Window w)
     }
 
 
-    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic) );
+    set_window_wm_state( tmp_win, get_flags(status.flags, AS_Iconic), from_map_request );
  
 	if( ASWIN_HFLAGS( tmp_win, AS_AvoidCover )  )
 		enforce_avoid_cover( tmp_win );
 
+
+	/* send_canvas_configure_notify(tmp_win->frame_canvas, tmp_win->client_canvas); */
+
     /*
 	 * Reparenting generates an UnmapNotify event, followed by a MapNotify.
 	 * Set the map state to FALSE to prevent a transition back to
@@ -289,9 +294,9 @@ AddWindow (Window w)
 	 * again in HandleMapNotify.
      */
     broadcast_config (M_ADD_WINDOW, tmp_win);
-    broadcast_window_name( tmp_win );
     broadcast_res_names( tmp_win );
     broadcast_icon_name( tmp_win );
+    broadcast_window_name( tmp_win );
 
     InstallWindowColormaps (tmp_win);
 	set_flags( tmp_win->internal_flags, ASWF_WindowComplete) ;
@@ -350,7 +355,7 @@ AddInternalWindow (Window w, ASInternalWindow **pinternal, ASHints **phints, ASS
 
     redecorate_window       ( tmp_win, False );
     on_window_title_changed ( tmp_win, False );
-    set_window_wm_state( tmp_win, get_flags(status->flags, AS_Iconic) );
+    set_window_wm_state( tmp_win, get_flags(status->flags, AS_Iconic), False );
 	RaiseWindow( tmp_win );
 /*    activate_aswindow( tmp_win, False, False ); */
 
@@ -361,9 +366,9 @@ AddInternalWindow (Window w, ASInternalWindow **pinternal, ASHints **phints, ASS
 	 * again in HandleMapNotify.
      */
     broadcast_config (M_ADD_WINDOW, tmp_win);
-    broadcast_window_name( tmp_win );
     broadcast_res_names( tmp_win );
     broadcast_icon_name( tmp_win );
+    broadcast_window_name( tmp_win );
 
 #if 0
 /* TODO : */
@@ -528,34 +533,38 @@ LOCAL_DEBUG_CALLER_OUT("%p, %d", asw, restart );
                startups are transparent for it */
             map_too = True ;
         }
-    }
-    /*
-     * Prevent the receipt of an UnmapNotify, since that would
-     * cause a transition to the Withdrawn state.
-     */
-    if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
-    {
-        XSelectInput (dpy, asw->w, NoEventMask);
-        if( get_parent_window( asw->w ) == asw->frame )
-        {
-            ASStatusHints withdrawn_state ;
-            /* !!! Most of it has been done in datach_basic_widget : */
-            XReparentWindow (dpy, asw->w, Scr.Root, x, y);
-            withdrawn_state.flags = AS_Withdrawn ;
-            withdrawn_state.icon_window = None ;
-            set_client_state( asw->w, &withdrawn_state );
+    	/*
+     	* Prevent the receipt of an UnmapNotify, since that would
+     	* cause a transition to the Withdrawn state.
+     	*/
+LOCAL_DEBUG_OUT ("map_too = %d", map_too );
 
-            if( width > 0 && height > 0 )
-                XResizeWindow( dpy, asw->w, width, height );
-            XSetWindowBorderWidth (dpy, asw->w, bw);
+    	if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
+    	{
+        	XSelectInput (dpy, asw->w, NoEventMask);
+        	if( get_parent_window( asw->w ) == asw->frame )
+        	{
+            	ASStatusHints withdrawn_state ;
+            	/* !!! Most of it has been done in datach_basic_widget : */
+            	XReparentWindow (dpy, asw->w, Scr.Root, x, y);
+            	withdrawn_state.flags = AS_Withdrawn ;
+            	withdrawn_state.icon_window = None ;
+            	set_client_state( asw->w, &withdrawn_state );
 
-            if( map_too )
-                XMapWindow (dpy, asw->w);
-            XSync( dpy, False );
-        }
-        /* signaling client that we no longer handle it : */
-        set_multi32bit_property (asw->w, _XA_WM_STATE, _XA_WM_STATE, 2, WithdrawnState, None);
-    }
+            	if( width > 0 && height > 0 )
+                	XResizeWindow( dpy, asw->w, width, height );
+            	XSetWindowBorderWidth (dpy, asw->w, bw);
+LOCAL_DEBUG_OUT ("map_too = %d", map_too );
+            	if( map_too )
+                	XMapWindow (dpy, asw->w);
+/*				else 
+					XUnmapWindow (dpy, asw->w); */
+            	XSync( dpy, False );
+        	}
+        	/* signaling client that we no longer handle it : */
+        	set_multi32bit_property (asw->w, _XA_WM_STATE, _XA_WM_STATE, 2, WithdrawnState, None);
+    	}
+	}
 }
 
 /**********************************************************************/
