@@ -29,6 +29,8 @@
 
 #include "../../configure.h"
 #include "../../libAfterStep/asapp.h"
+#include "../../libAfterStep/desktop_category.h"
+#include "../../libAfterConf/afterconf.h"
 #include "dirtree.h"
 void          init_func_data (FunctionData * data);
 int           txt2func (const char *text, FunctionData * fdata, int quiet);
@@ -53,9 +55,13 @@ dirtree_new (void)
 	return tree;
 }
 
+#define ASSERT_TREE(t)  do{if((t)==NULL){show_debug(__FILE__, __FUNCTION__,__LINE__,"dirtree is NULL%s","");return;}}while(0)
+#define ASSERT_TREE_INT(t,r)  do{if((t)==NULL){show_debug(__FILE__, __FUNCTION__,__LINE__,"dirtree is NULL%s","");return (r);}}while(0)
+
 void
 dirtree_remove (dirtree_t * tree)
 {
+	ASSERT_TREE(tree);
 	if (tree->parent != NULL && tree->parent->child != NULL)
 	{
 		if (tree->parent->child == tree)
@@ -78,6 +84,7 @@ dirtree_remove (dirtree_t * tree)
 void
 dirtree_delete (dirtree_t * tree)
 {
+	ASSERT_TREE(tree);
 	/* find and remove ourself from our parent's list */
 	dirtree_remove (tree);
 
@@ -86,10 +93,10 @@ dirtree_delete (dirtree_t * tree)
 		dirtree_delete (tree->child);
 
 	/* free members */
-	if (tree->name != NULL)
-		free (tree->name);
 	if (tree->stripped_name != NULL && tree->stripped_name != tree->name )
 		free (tree->stripped_name);
+	if (tree->name != NULL)
+		free (tree->name);
 	if (tree->path != NULL)
 		free (tree->path);
 	if (tree->icon != NULL)
@@ -99,6 +106,12 @@ dirtree_delete (dirtree_t * tree)
 	if (tree->minipixmap_extension != NULL)
 		free (tree->minipixmap_extension);
 	free_func_data (&tree->command);
+	if (tree->de != NULL)
+		unref_desktop_entry(tree->de);
+	if (tree->Comment != NULL)
+		free (tree->Comment);
+	if (tree->FolderReference != NULL)
+		free (tree->FolderReference);
 
 	free (tree);
 }
@@ -108,8 +121,10 @@ make_absolute (const char *path1, const char *path2)
 {
 	char         *path;
 
-	if (*path2 == '/' || *path2 == '~')
-		path = PutHome (path2);
+	if( path1 == NULL || path2 == NULL )
+		return mystrdup("./") ;
+	if (*path2 == '/' || *path2 == '~' || *path2 == '$')
+		path = copy_replace_envvar(path2);
 	else
 		/* relative path */
 	{
@@ -143,7 +158,8 @@ dirtree_fill_from_dir (dirtree_t * tree)
 {
 	struct direntry **list;
 	int           i, n;
-
+	
+	ASSERT_TREE(tree);
 	n = my_scandir (tree->path, &list, no_dots_except_include, NULL);
 	for (i = 0; i < n; i++)
 	{
@@ -172,33 +188,38 @@ dirtree_fill_from_dir (dirtree_t * tree)
 dirtree_t    *
 dirtree_new_from_dir (const char *dir)
 {
-	dirtree_t    *tree = dirtree_new ();
-	char   *p;
-	int start_mark = 0, end_mark ;
-	register int i = 0 ;
-	register char *ptr = (char*)dir ;
+	
+	dirtree_t    *tree = NULL ; 
 
-	while( ptr[i] ) ++i ;
-	end_mark = i ;
+	if( dir )
+	{	
+		char   *p;
+		int start_mark = 0, end_mark ;
+		register int i = 0 ;
+		register char *ptr = (char*)dir ;
+		tree = dirtree_new ();
+	
+		while( ptr[i] ) ++i ;
+		end_mark = i ;
 
-	p = tree->path = safemalloc( i+1 );
-	do{ p[i] = ptr[i]; } while( --i >= 0 );
-	while( --i > 0 )
-		if( ptr[i] =='/' )
-		{
-			end_mark = i ;
-			break ;
-		}
-
-	while( --i >= 0 )
-		if( ptr[i] =='/' )
-		{
-			start_mark = i+1 ;
-			break ;
-		}
-	tree->name = mystrndup (&(ptr[start_mark]), end_mark-start_mark);
-	dirtree_fill_from_dir (tree);
+		p = tree->path = safemalloc( i+1 );
+		do{ p[i] = ptr[i]; } while( --i >= 0 );
+		while( --i > 0 )
+			if( ptr[i] =='/' )
+			{
+				end_mark = i ;
+				break ;
+			}
 
+		while( --i >= 0 )
+			if( ptr[i] =='/' )
+			{
+				start_mark = i+1 ;
+				break ;
+			}
+		tree->name = mystrndup (&(ptr[start_mark]), end_mark-start_mark);
+		dirtree_fill_from_dir (tree);
+	}
 	return tree;
 }
 
@@ -206,6 +227,8 @@ dirtree_new_from_dir (const char *dir)
 void
 dirtree_move_children (dirtree_t * tree1, dirtree_t * tree2)
 {
+	ASSERT_TREE(tree1);
+	ASSERT_TREE(tree2);
 	if (tree2->child != NULL)
 	{
 		dirtree_t    *t1, *t2 = NULL;
@@ -225,25 +248,159 @@ void
 dirtree_set_command (dirtree_t * tree, struct FunctionData *command, int recurse)
 {
 	dirtree_t    *t;
-
-	for (t = tree->child; t != NULL; t = t->next)
-	{
-		t->command = *command;
-		if (t->command.text != NULL)
-			t->command.text = mystrdup (t->command.text);
-		if (recurse)
-			dirtree_set_command (t, command, 1);
+	ASSERT_TREE(tree);
+	if( command )
+	{	
+		for (t = tree->child; t != NULL; t = t->next)
+		{
+			t->command = *command;
+			if (t->command.text != NULL)
+				t->command.text = mystrdup (t->command.text);
+			if (recurse)
+				dirtree_set_command (t, command, 1);
+		}
 	}
 }
 
+void 
+dirtree_add_category (dirtree_t *tree, ASCategoryTree *ct, ASDesktopCategory *dc, Bool include_children, ASHashTable *exclusions)
+{
+	dirtree_t    *t = NULL ; 
+	int i, entries_num, valid_entries_num = 0  ;
+	char **entries_names ; 
+	struct ASDesktopEntryInfo { ASDesktopEntry *de; ASDesktopCategory *dc ; } *entries ; 
+	
+
+	ASSERT_TREE(tree);
+	ASSERT_TREE(dc);
+	ASSERT_TREE(ct);
+
+	entries_num = PVECTOR_USED(dc->entries);
+	LOCAL_DEBUG_OUT( "DesktopCategory \"%s\", has %d entries", dc->name, entries_num );
+	if( entries_num == 0 ) 
+		return;
+	entries_names = PVECTOR_HEAD(char*, dc->entries );
+	entries = safecalloc( entries_num, sizeof( struct ASDesktopEntryInfo ) );
+
+	for( i = 0 ; i < entries_num ; ++i ) 
+	{
+		ASDesktopEntry *de;
+		if( exclusions ) 
+			if( get_hash_item( exclusions, AS_HASHABLE(entries_names[i]), NULL ) == ASH_Success ) 
+				continue;
+				
+		de = fetch_desktop_entry( ct, entries_names[i] ); 
+		LOCAL_DEBUG_OUT( "entry \"%s\", de = %p", entries_names[i], de );
+		if( de == NULL ) 
+			continue;
+		if( get_flags(de->flags, ASDE_Hidden|ASDE_NoDisplay) ) 
+			continue;
+		if( de->type == ASDE_TypeDirectory )
+		{
+			if( !include_children ) 
+				continue;
+			entries[valid_entries_num].dc = fetch_desktop_category( ct, de->IndexName?de->IndexName:de->Name );
+			if( entries[valid_entries_num].dc == NULL || PVECTOR_USED(entries[valid_entries_num].dc->entries ) == 0 ) 
+				continue;
+		}				
+		entries[valid_entries_num].de = de ; 
+		++valid_entries_num ;
+	}
+		
+	for( i = 0 ; i < valid_entries_num ; ++i ) 
+	{
+		ASDesktopEntry *de = entries[i].de;
+		ASDesktopCategory *sub_dc = entries[i].dc ; 
+
+		if( de->type != ASDE_TypeDirectory && include_children )
+		{
+			int k ;
+			for( k = 0 ; k < valid_entries_num ; ++k ) 
+				if( entries[k].de->type == ASDE_TypeDirectory )
+					if( desktop_entry_belongs_to( ct, de, entries[k].de ) ) 
+						break;
+			if( k < valid_entries_num ) 
+				continue; /* will be included in one of our children */ 	
+		}
+		
+		t = dirtree_new ();
+		
+		if( dup_desktop_entry_Name( de, &(t->name) ) )
+			set_flags( t->flags, DIRTREE_NAME_IS_UTF8 );
+
+		if( dup_desktop_entry_Comment( de, &(t->Comment) ) )
+			set_flags( t->flags, DIRTREE_COMMENT_IS_UTF8 );
+		
+		ref_desktop_entry( de );
+		t->de = de ; 
+		
+		t->parent = tree;
+		t->next = tree->child;
+		tree->child = t;			
+		
+		if( sub_dc ) 
+		{				   
+			t->flags |= DIRTREE_DIR;
+			dirtree_add_category (t, ct, sub_dc, include_children, exclusions);
+		}	 
+	}	
+	free( entries ) ;
+}
+
+
+void 
+dirtree_add_category_by_name (dirtree_t *tree, const char *cat_name, Bool include_children, ASHashTable *exclusions)
+{
+	ASCategoryTree *ct = CombinedCategories ; 
+	ASDesktopCategory *dc = NULL ;
+	
+	ASSERT_TREE(tree);
+
+	dc = name2desktop_category( cat_name, &ct ); 
+	dirtree_add_category (tree, ct, dc, include_children, exclusions);
+}
+
+void dirtree_fill_from_reference( dirtree_t *tree, const char *reference )
+{
+	ASCategoryTree *ct = CombinedCategories ; 
+	ASDesktopCategory *dc = NULL ;
+	ASDesktopEntry *de = NULL ; 
+	
+	ASSERT_TREE(tree);
+
+	dc = name2desktop_category( reference, &ct ); 
+	de = fetch_desktop_entry( ct, dc->index_name?dc->index_name:dc->name ); 
+
+	if( dup_desktop_entry_Name( de, &(tree->name) ) )
+		set_flags( tree->flags, DIRTREE_NAME_IS_UTF8 );
+
+	if( dup_desktop_entry_Comment( de, &(tree->Comment) ) )
+		set_flags( tree->flags, DIRTREE_COMMENT_IS_UTF8 );
+
+	if( de->fulliconname )
+	{
+		set_string(&(tree->icon), mystrdup(de->fulliconname));
+		set_flags(tree->flags, DIRTREE_ICON_IS_SMALL);
+	}
+}
+
+
 int
 dirtree_parse (dirtree_t * tree, const char *file)
 {
 	FILE         *fp;
 	char         *str;
+	ASHashTable  *exclusions = NULL; 
 
+	ASSERT_TREE_INT(tree,1);
+	
+	if( file == NULL ) 
+		return 1 ;
+	
 	if ((fp = fopen (file, "r")) == NULL)
 		return 1;
+	
+	
 	str = safemalloc (8192);
 	while (fgets (str, 8192, fp) != NULL)
 	{
@@ -255,13 +412,44 @@ dirtree_parse (dirtree_t * tree, const char *file)
 		/* ignore comments and blank lines */
 		if (*ptr == '#' || *ptr == '\0')
 			continue;
+		if( !mystrncasecmp (ptr, "exclude", 7) )
+		{
+			char *excl_name ; 
+			if( exclusions == NULL ) 
+				exclusions = create_ashash( 0, casestring_hash_value, casestring_compare, string_destroy );
+			if( exclusions ) 
+			{	
+				excl_name = stripcpy2 (ptr +7, 0);
+				add_hash_item( exclusions, AS_HASHABLE(excl_name), NULL );
+			}
+			continue;
+		}		
+		
+		if( !mystrncasecmp (ptr, "category", 8) )
+		{
+			char *cat_name;
+			Bool include_children = False;
+			ptr+= 8 ;
+			if( *ptr == '_' ) ++ptr ;
+			if( !mystrncasecmp (ptr, "tree", 4))
+			{
+				include_children = True ;
+				ptr += 4 ;	  
+			}
+			cat_name = stripcpy2 (ptr, 0);
+			dirtree_add_category_by_name (tree, cat_name, include_children, exclusions);
+			free( cat_name );
+			continue;
+		}
+		   
 		if( !mystrncasecmp (ptr, "include", 7) )
 		{
 			do_include = True ;
 			ptr += 7 ; 
-			if( !mystrncasecmp (ptr, "_ordered", 8) ) 	  
+			if( *ptr == '_' ) ++ptr ;
+			if( !mystrncasecmp (ptr, "ordered", 7) ) 	  
 			{
-				for (ptr += 8; isspace (*ptr); ptr++);				
+				for (ptr += 7; isspace (*ptr); ptr++);				
 				if ( isdigit(*ptr) )
 				{
 					include_order = atoi( ptr );	
@@ -303,7 +491,9 @@ dirtree_parse (dirtree_t * tree, const char *file)
 			}
 		} else if (!mystrncasecmp (ptr, "keepname", 8))
 			tree->flags |= DIRTREE_KEEPNAME;
-		else if (!mystrncasecmp (ptr, "extension", 9))
+		else   if (!mystrncasecmp (ptr, "ShowUnavailable", 15))
+			tree->flags |= DIRTREE_SHOW_UNAVAILABLE;
+		   else if (!mystrncasecmp (ptr, "extension", 9))
 		{
 			char         *tmp;
 
@@ -322,12 +512,16 @@ dirtree_parse (dirtree_t * tree, const char *file)
 			for (tmp = ptr + strlen (ptr); tmp > ptr && isspace (*(tmp - 1)); tmp--);
 			if (tmp != ptr)
 				tree->minipixmap_extension = mystrndup (ptr, tmp - ptr);
-		} else if (!mystrncasecmp (ptr, "minipixmap", 10))
+		} else if (!mystrncasecmp (ptr, "minipixmap", 10) || !mystrncasecmp (ptr, "smallminipixmap", 15))
 		{
-			for (ptr += 10; isspace (*ptr); ptr++);
-			if (tree->icon != NULL)
-				free (tree->icon);
-			tree->icon = mystrdup (ptr);
+			if( ptr[0] == 's' || ptr[0] == 'S' )
+			{	
+				set_flags(tree->flags, DIRTREE_ICON_IS_SMALL);
+				ptr += 5 ;
+			}else
+				clear_flags(tree->flags, DIRTREE_ICON_IS_SMALL);
+			set_string(&(tree->icon), stripcpy2(ptr+10,False));
+
 		} else if (!mystrncasecmp (ptr, "command", 7))
 		{
 			for (ptr += 7; isspace (*ptr); ptr++);
@@ -342,10 +536,16 @@ dirtree_parse (dirtree_t * tree, const char *file)
 			tree->flags |= DIRTREE_RECENT_ITEMS_SET;
 		} else if (!mystrncasecmp (ptr, "name", 4))
 		{
-			for (ptr += 4; isspace (*ptr); ptr++);
-			if (tree->name != NULL)
-				free (tree->name);
-			tree->name = mystrdup (ptr);
+			set_string(&(tree->name), stripcpy2(ptr+4,False));
+			clear_flags( tree->flags, DIRTREE_NAME_IS_UTF8 );
+		}else if (!mystrncasecmp (ptr, "Comment", 7))
+		{
+			set_string(&(tree->Comment), stripcpy2(ptr+7,False));
+			clear_flags( tree->flags, DIRTREE_COMMENT_IS_UTF8 );
+		}else if (!mystrncasecmp (ptr, "FolderReference", 15))
+		{
+			set_string(&(tree->FolderReference), stripcpy2(ptr+15,False));
+			dirtree_fill_from_reference( tree, tree->FolderReference );
 		}
 	}
 	free (str);
@@ -357,6 +557,7 @@ void
 dirtree_parse_include (dirtree_t * tree)
 {
 	dirtree_t    *t;
+	ASSERT_TREE(tree);
 
 	/* parse the first .include */
 	for (t = tree->child; t != NULL; t = t->next)
@@ -381,6 +582,7 @@ void
 dirtree_set_base_order (dirtree_t * tree, int base_order)
 {
 	dirtree_t    *t;
+	ASSERT_TREE(tree);
 	
 	if( tree->base_order == 0 ) 
 		tree->base_order = base_order ; 
@@ -403,6 +605,7 @@ dirtree_remove_order (dirtree_t * tree)
 	dirtree_t    *t;
 	char         *ptr;
 	int           order = strtol (tree->name, &ptr, 10);
+	ASSERT_TREE(tree);
 
 	if (ptr != tree->name && *ptr == '_')
 	{
@@ -417,6 +620,8 @@ void
 dirtree_merge (dirtree_t * tree)
 {
 	dirtree_t    *t;
+	ASSERT_TREE(tree);
+	
 	/* PASS1: merge all the subdirs of the current dir */
 	for (t = tree->child; t != NULL; t = t->next)
 	{
@@ -480,6 +685,9 @@ dirtree_compar (const dirtree_t ** d1, const dirtree_t ** d2)
 	int           diff = 0;
 	dirtree_compar_f *compar;
 
+	ASSERT_TREE_INT(*d1,1);
+	ASSERT_TREE_INT(*d2,-1);
+
 	for (compar = dirtree_compar_list; diff == 0 && *compar != NULL; compar++)
 		diff = (*compar) (d1, d2);
 	return diff;
@@ -528,6 +736,8 @@ dirtree_sort (dirtree_t * tree)
 	dirtree_t    *t;
 	dirtree_t   **list;
 
+	ASSERT_TREE(tree);
+
 	if (tree->child == NULL)
 		return;
 
@@ -665,7 +875,7 @@ void
 dirtree_print_tree (dirtree_t * tree, int depth)
 {
 	dirtree_t    *t;
-
+	ASSERT_TREE(tree);
 	fprintf (stderr, "%*s%s%s(%s: order = %d: base_order = %d: flags = %x)\n", depth, "", tree->name,
 			 (tree->flags & DIRTREE_DIR) ? "/ " : " ", tree->icon,
 			 tree->order, tree->base_order, tree->flags);
